/**
 * Fichier rsd.c
 *
 * Ce fichier fait parti du logiciel Remote Sniffers Detector (RSD)
 * et est soumis ala licence gpl, disponible sur www.gnu.org/licenses/gpl.txt
 *
 * Auteur : Henri Doreau <henri.doreau@gmail.com>
 * Date : 23 aout 2007
 *
 * Version : 0.1alpha
 **/

#include "rsd.h"


int main(int argc, char *argv[])
{
	char opt, mac[6]; /* opt est le code de sortie de getopt(), mac[] designe l'adresse mac locale */
	char *target, *interface; /* adresse IP de la cible, sous la forme xxx.xxx.xxx.xxx et interface choisie */
	struct sockaddr_in t_addr; /* structure de definition de la cible */
	int _socket, card_index; /* descripteur de fichier de la socket et index de l'interface */
	struct sockaddr_ll device; /* structure de definition de la carte reseau utilisee */

	if (check(argc, argv) < 0)
			return -1;

	/* Parsing des arguments et affectation de l'IP cible a la structure t_addr */
	while ((opt = getopt(argc, argv, "t:i:h")) != -1)
	{
		switch(opt) 
		{
    		case 't':	target = hostInfo(optarg, &t_addr);
						break;

			case 'i':	interface = optarg;
						break;

			case 'h':	help();
						return -1;

			default:	printf("%s -t <target IPV4 address> -i <interface>\n", argv[0]);
						return -1;
		}
	}

#ifdef DEBUG
	fprintf(stdout,"Target address set to : %s\n",target);
#endif
	if ((_socket = socket(PF_PACKET, SOCK_RAW, htons(ETH_P_ALL))) < 0) /* ouverture de la socket */
	{
		perror("socket");
		return -1;
	}
	memset(&device, 0, sizeof(device)); /* initialisation de la structure */
	localMac(_socket, interface, mac, &device.sll_ifindex); /* recherche de l'adresse mac locale */ 
#ifdef DEBUG
	fprintf(stdout,"Interface %s selected\n", interface);
#endif
	device.sll_family = AF_PACKET; /* toujours ainsi d'apres les pages de man, faisons confiance... */
	memcpy(device.sll_addr, mac, 6); /* copie de l'adresse mac locale dans la structure de definition de la carte */
	device.sll_halen = htons(6); /* longueur de l'adresse MAC */
  	craftAndSend(_socket, *(unsigned int *)&t_addr.sin_addr, mac, &device); /* conception et envoi du paquet requete */
	parseAnswer(_socket, &device, target);

	return 0;
}


void help(void)
{
	/**
	 * Cette procdure affiche simplement l'aide
	 * la version et le about du programme.
	 *
	 * Elle est appele par l'option -h et engendre la fin du programme
	 **/
	
	printf("\n-=] Remote Sniffers Detector [=-\n");
	printf("              ****          \n");
	printf("\nhttp://hatch-the-hitch.blogspot.com\n");
	printf("\nVersion : %s\n", VERSION);
	printf("Usage : ./rsd -t <target IP V4 address> -i <interface>\n");
	printf("For example : ./rsd -t 192.168.1.19 -i eth0\n\n");
	return;
}


char *hostInfo(char *host, struct sockaddr_in *t_addr)
{
	struct addrinfo hints, *res;

	/**
	 * Prends en argument une adresse/nom et un pointeur vers une structure
	 * et remplis la structure avec les informations concernant la cible
	 **/
	memset(&hints, 0, sizeof(hints));
	hints.ai_family = PF_INET;
	getaddrinfo(host, NULL, &hints, &res);
	memcpy(t_addr, res->ai_addr, res->ai_addrlen);

	return inet_ntoa(t_addr->sin_addr);
}


char localMac(int _socket, char *iface, char *mac_addr, int *card_index)
{
	struct ifreq card;

	/**
	 * Utilisation de la fonction ioctl
	 * afin de retrouver l'adresse MAC locale
	 **/
	strcpy(card.ifr_name, iface); /* iface == nom de l'interface utilisee */

	if (ioctl(_socket, SIOCGIFHWADDR, &card) == -1) /* adresse materielle (MAC) */
	{
		perror("ioctl");
		return -1;
	}

	memcpy(mac_addr, card.ifr_hwaddr.sa_data, 6); /* copie de cette adresse dans mac (declare dans main()) */

	if (ioctl(_socket, SIOCGIFINDEX, &card) == -1) /* index de la carte */
	{
		perror("ioctl");
		return -1;
	}
	*card_index = card.ifr_ifindex;
	return 0;
}


char craftAndSend(int _socket, unsigned int tgt_ip, char *local_mac, struct sockaddr_ll *device)
{
	ether_packet trame;
	struct in_addr local_ip;

#ifdef DEBUG
	fprintf(stdout, "Crafting packet\n");
#endif
	/**
	 * Conception du paquet
	 *
	 * les structures sont declarees dans le fichier rsd.h
	 **/
	inet_aton(LOCAL_ADDR, &local_ip);
	memset(trame.dest_mac, 0xFF, 5);
	trame.dest_mac[5] = 0xFE;
	memcpy(trame.sender_mac, local_mac, 6);
	trame.eth_type = htons(0x806);
	trame._packet.hw_type = htons(0x1);
	trame._packet.proto_type = htons(0x800);
	trame._packet.hw_len = 0x6;
	trame._packet.proto_len = 0x4;
	trame._packet.opcode = htons(0x1);
	memcpy(trame._packet.src_mac, local_mac, 6);
	memcpy(trame._packet.src_ip, (char *)&local_ip, 4);
	memset(trame._packet.target_mac, 0xFF, 5);
	trame._packet.target_mac[5] = 0xFE;
	memcpy(trame._packet.target_ip, (char *)&tgt_ip, 4);
#ifdef DEBUG
	fprintf(stdout, "Sending request\n");
#endif
	/**
	 * Envoi du paquet DIRECTEMENT AU DRIVER ==> sans aucune encapsulation
	 * la fonction sendto() renvoit le nombre d'octet transmis, ou une valeur negative en cas d'erreur
	 **/
	if (sendto(_socket, &trame, sizeof(trame), 0, (const struct sockaddr *)device, sizeof(*device)) <= 0)
	{
		perror("Sending");
		return -1;
	}
	return 0;
}


void parseAnswer(int _socket, struct sockaddr_ll *device, char *target)
{
	ether_packet reply;
	struct in_addr reply_src_ip;

	memset(&reply, 0, sizeof(reply));
	inet_aton(target, &reply_src_ip);
#ifdef DEBUG
	fprintf(stdout,"En attente d'une reponse\n");
#endif
	/**
	 * Si le programme ne recoit pas de reponse
	 * apres le timeout du protocole, le programme quitte automatiquement
	 **/
	recvfrom(_socket, &reply, sizeof(reply), 0, (struct sockaddr *)device, (unsigned int *)sizeof(*device));

	/*
	 * Verifie que l'emetteur est bien la cible et qu'il s'agit bien d'un paquet reply
	 */
	if ( (memcmp(reply._packet.src_ip, (char *)&reply_src_ip, 4) == 0) || (reply._packet.opcode == htons(2)) )
			printf("La cible %s a repondu!\n", target);
	return;
}

char check(int argc, char *argv[])
{
	/**
	 * Cette fonction, appelee en debut de programme effectue quelques tests de base
	 * afin de verifier qu'on a le minimum pour poursuivre
	 **/
	if ( (argc < 5) && (argc != 2) )
	{
		printf("Bad options line\ntry %s -h for help\n", argv[0]);
		return -1;
	}
	if (!IS_ROOT)
	{
		printf("Vous devez etre root pour utiliser %s\n", argv[0]);
		return -1;
	}
	return 0;
}

