/**
 * Fichier rsd.h
 *
 * Ce fichier fait parti du logiciel Remote Sniffers Detector (RSD)
 * et est soumis a la licence gpl, disponible sur www.gnu.org/licenses/gpl.txt
 *
 * Auteur : Henri Doreau <henri.doreau@gmail.com>
 * Date : 23 aout 2007
 *
 * Version : 0.1alpha
 **/


/* ------ NETWORK LIBS ------ */
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <net/if.h>
#include <linux/if_ether.h>
#include <netpacket/packet.h>
#include <net/ethernet.h>

/* ------ SYSTEM LIBS ------ */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <errno.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <stddef.h>


#define VERSION "0.1 alpha"
#define LOCAL_ADDR "192.168.1.12"

/* Macro */
#define IS_ROOT ( ((getuid() != 0) || (geteuid() != 0)) ? 0 : 1 )


extern int optind, opterr, optopt;
extern char *optarg;

/**
 * Paquet ARP
 *
 * Les noms des champs sont explicites
 * pour plus d'informations : http://www.networksorcery.com/enp/protocol/arp.htm
 **/
typedef struct {
	unsigned short hw_type;
	unsigned short proto_type;
	unsigned char  hw_len;
	unsigned char  proto_len;
	unsigned short opcode;
	unsigned char  src_mac[6];
	unsigned char  src_ip[4];
	unsigned char  target_mac[6];
	unsigned char  target_ip[4];
} _packet;

/*
 * Trame Ethernet complète (IPV4 ARP packet)
 */
typedef struct {
	unsigned char  dest_mac[6];
	unsigned char  sender_mac[6];
	unsigned short eth_type;
	_packet     _packet;
} ether_packet;


/* Banal affichage de l'aide */
void help(void);
/* Verification des conditions nécessaires à la bonne execution du programme */
char check(int argc, char *argv[]);
/* Informations sur l'hote choisi */
char *hostInfo(char *host, struct sockaddr_in *t_addr);
/* le programme se charge de retrouver l'adresse MAC locale */
char localMac(int _socket, char *iface, char *mac_addr, int *card_index);
/* Construction et envoi de la requete */
char craftAndSend(int _socket, unsigned int tgt_ip, char *local_mac, struct sockaddr_ll *device);
/* Attend un paquet ARP reply et, si en recoit parse le champs ip_source*/
void parseAnswer(int _socket, struct sockaddr_ll *device, char *target);


/**
 * ===============================
 *		 STRUCTURES UTILISEES
 * ===============================
 *
 * struct sockaddr {
 *   sa_family_t  sa_family;	address family, AF_xxx
 *   char		  sa_data[14];	14 bytes of protocol address
 * };
 *
 *
 * struct sockaddr_ll {
 *  unsigned short  sll_family;    Always AF_PACKET
 *  unsigned short  sll_protocol;  Physical layer protocol
 *  int             sll_ifindex;   Interface number
 *	unsigned short  sll_hatype;    Header type
 * 	unsigned char   sll_pkttype;   Packet type
 * 	unsigned char   sll_halen;     Length of address
 * 	unsigned char   sll_addr[8];   Physical layer address
 * };
 **/

